# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/ftpImport/DmsFtp.py $
# Last committed:     $Revision: 194224 $; $Author: svcdms $; $Date: 2019-07-01 11:42:03 +0200 (ma, 01 jul 2019) $
#
from datetime import datetime
from dateutil import parser
from errno import EEXIST
from ftplib import FTP, error_perm
from misc.Utils import Utils
from os import path, makedirs
from re import sub
from sys import setrecursionlimit


class DmsFtp:
    setrecursionlimit(3000)

    def __init__(self, logger, db):
        self._logger = logger
        self._site = Utils.read_ftp_settings()
        self._db = db
        try:
            self._ftp = FTP(
                host=self._site["url"], user=self._site["usr"], passwd=self._site["pw"]
            )
        except ConnectionRefusedError:
            logger.error(
                "DmsFtp.init: could not log in to ftp site, site not available"
            )
            exit(1)
        except error_perm:
            logger.error(
                "DmsFtp.init: could not log in to ftp site, login or password incorrect"
            )
            exit(1)
        else:
            self._logger.debug("DmsFtp: welcome {}".format(self._ftp.getwelcome()))

    def get_timestamp_of_files(self, remote_folder):
        self._logger.info("DmsFtp.get_timestamp_of_files ({})".format(remote_folder))
        timestamps = {}
        for file in self._ftp.mlsd(remote_folder):
            # self._logger.debug("DmsFtp.get_timestamp_of_files ({})".format(file[0]))
            if file[1]["type"] == "file":
                timestamps[file[0]] = parser.parse(file[1]["modify"])
        return timestamps

    def do_bin_ftp(self, remote_folder, file, local_file):
        self._logger.debug("DmsFtp.do_bin_ftp({},{})".format(file, local_file))
        remote_file = remote_folder + "/" + file

        # create folder if it does not exist
        directory = path.dirname(local_file)
        try:
            makedirs(directory)
            self._logger.info("DmsFtp.do_bin_ftp: created folder {}".format(directory))
        except (OSError, FileExistsError) as e:
            if e.errno != EEXIST:
                self._logger.error(
                    "DmsFtp.do_bin_ftp: could not create folder {}".format(directory)
                )
                raise
        # start downloading the file
        try:
            self._logger.debug(
                "DmsFtp.do_bin_ftp: downloading: {} --> {}".format(
                    remote_file, local_file
                )
            )
            # this will raise an exception in case the file is not found
            size = self._ftp.size(remote_file)
            with open(local_file, "wb") as f:
                self._ftp.retrbinary("RETR {}".format(remote_file), f.write)
                return True
        except (OSError, error_perm):
            self._db.add_file_to_list(
                remote_folder,
                directory,
                file,
                "ftp_download",
                datetime.now(),
                datetime.now(),
                1,
            )
            self._logger.error(
                "DmsFtp.do_bin_ftp: problem downloading file {} to {}".format(
                    remote_file, local_file
                )
            )
            return False

    def close_connection(self):
        self._logger.debug("close_connection")
        self._ftp.quit()

    def recurse_ftp_tree(self, root, tree):
        for d in self._ftp.mlsd(root):
            # exclude test folder
            if d[0] == "FL0":
                continue
            if d[1]["type"] == "dir":
                p = sub("//", "/", root + "/" + d[0])
                tree.append(p)
                tree = self.recurse_ftp_tree(p, tree)
        return tree
